qpakman lq_dev/*.png -o lq_dev.wad
qpakman lq_dev_legacy/*.png -o lq_dev_legacy.wad
qpakman lq_flesh/*.png -o lq_flesh.wad
qpakman lq_greek/*.png -o lq_greek.wad
qpakman lq_health_ammo/*.png -o lq_health_ammo.wad
qpakman lq_legacy/*.png -o lq_legacy.wad
qpakman lq_liquidsky/*.png -o lq_liquidsky.wad
qpakman lq_mayan/*.png -o lq_mayan.wad
qpakman lq_medieval/*.png -o lq_medieval.wad
qpakman lq_metal/*.png -o lq_metal.wad
qpakman lq_palette/*.png -o lq_palette.wad
qpakman lq_props/*.png -o lq_props.wad
qpakman lq_tech/*.png -o lq_tech.wad
qpakman lq_terra/*.png -o lq_terra.wad
qpakman lq_utility/*.png -o lq_utility.wad
qpakman lq_wood/*.png -o lq_wood.wad
qpakman lq_conc/*.png -o lq_conc.wad

